use std::collections;
use std::io::Read;

pub fn startup(auth: &collections::HashMap<String, String>) -> (curl::easy::Easy, String) {
    let url = auth.get("ha_url").expect("[ERROR] Failed to read the URL").as_str();

    // escape character for { is {{ and for } is }}
    let entity_id = auth.get("light_entity_id").expect("[ERROR] Failed to read the entity id").as_str();
    let data = format!("{{\"entity_id\": {entity_id}}}");
    
    let token = auth.get("access_token").expect("[ERROR] Failed to read the token").as_str();
    let auth = format!("Authorization: Bearer {}", &token[1..token.len()-1]);

    let mut list = curl::easy::List::new();
    list.append(&auth).unwrap();
    list.append("Content-Type: application/json").unwrap();

    let mut request = curl::easy::Easy::new();

    request.url(&url[1..url.len()-1]).expect("[ERROR] Failed to set the URL for the CURL request");
    request.post(true).expect("[ERROR] Failed to set the CURL request as POST");
    request.http_headers(list).expect("ERROR] Failed to apply HTTP headers");
    request.post_field_size(data.len() as u64).expect("[ERROR] Failed to set the POST field size for CURL request");

    (request, data)
}
pub fn request(request: &mut curl::easy::Easy, data: &str) -> Option<u32> {
    let mut transfer = request.transfer();
    
    if let Err(e) = transfer.read_function(|buf| {
        Ok(data.as_bytes().read(buf).unwrap_or(0))
    }) {
        eprintln!("[ERROR] Failed to read the data:\n{e}");
        return None;
    }

    if let Err(e) = transfer.perform() {
        eprintln!("[ERROR] Failed to perform the CURL request:\n{e}");
        return None;
    }
    
    // drop transfer so we can access request.response_code below
    drop(transfer);
    
    match request.response_code() { 
        Ok(t) => Some(t),
        Err(e) => {
            eprintln!("[ERROR] Failed to retrieve the response code:\n{e}");
            None
        },
    }
}